@@ -1,13 +1,19 @@
 """
 The interactive shell interface to the HERA debugger.
 
-Author:  Ian Fisher (iafisher@protonmail.com)
+Author:  Ian Fisher (iafisher@fastmail.com)
 Version: March 2019
 """
 import copy
 import functools
 import textwrap
-from typing import List, Optional
+
+from hera.assembler import assemble_and_print
+from hera.data import DataLabel, HERAError, Label, Location, Program, Settings
+from hera.loader import load_program
+from hera.op import LABEL, OPCODE, Branch, DataOperation, disassemble, name_to_class
+from hera.parser import parse
+from hera.utils import format_int, out_of_range, pad
 
 from . import miniparser
 from .debugger import Debugger
@@ -20,12 +26,6 @@ from .miniparser import (
     RegisterNode,
     SymbolNode,
 )
-from hera.assembler import assemble_and_print
-from hera.data import DataLabel, HERAError, Label, Location, Program, Settings
-from hera.loader import load_program
-from hera.op import Branch, DataOperation, disassemble, LABEL, name_to_class, OPCODE
-from hera.parser import parse
-from hera.utils import format_int, out_of_range, pad
 
 
 def debug(program: Program, settings: Settings) -> None:
@@ -77,8 +77,16 @@ class Shell:
             print("Cannot debug an empty program.")
             return
 
+        print("Welcome to the HERA debugger.")
+        print()
+        print("Enter 'help' to see a list of valid commands.")
+        print("Enter 'quit' or press Ctrl+D to exit.")
+        print()
+        print()
+
         self.print_current_op()
 
+        previous = None
         while True:
             try:
                 response = input(">>> ").strip()
@@ -87,11 +95,18 @@ class Shell:
                 break
 
             if not response:
-                continue
-            else:
-                should_continue = self.handle_command(response)
-                if should_continue is False:
-                    break
+                if previous:
+                    response = previous
+                    print("(executing previous command: {})".format(previous))
+                    print()
+                else:
+                    continue
+
+            should_continue = self.handle_command(response)
+            if should_continue is False:
+                break
+
+            previous = response
 
     def handle_command(self, response: str) -> bool:
         """
@@ -184,7 +199,7 @@ class Shell:
         assemble_and_print(program, settings)
 
     @mutates
-    def handle_assign(self, args: List[str]) -> None:
+    def handle_assign(self, args: "List[str]") -> None:
         """
         assign <x> <y>
           Assign the value of y to x. x may be a register, a memory location, or the
@@ -250,7 +265,7 @@ class Shell:
             print("Eval error: " + str(e) + ".")
 
     @mutates
-    def handle_break(self, args: List[str]) -> None:
+    def handle_break(self, args: "List[str]") -> None:
         """
         break
           Print all current breakpoints.
@@ -290,7 +305,7 @@ class Shell:
                 print("Breakpoint set in file {0.path}, line {0.line}.".format(loc))
 
     @mutates
-    def handle_clear(self, args: List[str]) -> None:
+    def handle_clear(self, args: "List[str]") -> None:
         """
         clear <location>
           Clear a breakpoint at the given line. Location formats accepted are the same
@@ -322,7 +337,7 @@ class Shell:
                         print("No breakpoint at that location.")
 
     @mutates
-    def handle_continue(self, args: List[str]) -> None:
+    def handle_continue(self, args: "List[str]") -> None:
         """
         continue
           Execute the program until a breakpoint is encountered or the program
@@ -338,7 +353,7 @@ class Shell:
 
         self.print_current_op()
 
-    def handle_dis(self, args: List[str]) -> None:
+    def handle_dis(self, args: "List[str]") -> None:
         """
         dis <n>
           Interpret the 16-bit integer as a HERA machine instruction, and disassemble
@@ -404,7 +419,7 @@ class Shell:
         labels, and avoid using relative branching instructions at all.
     """
 
-    def handle_doc(self, args: List[str]) -> None:
+    def handle_doc(self, args: "List[str]") -> None:
         """
         doc <opname>...
           For each operation, print a message detailing its use and behavior.
@@ -476,7 +491,7 @@ class Shell:
         vm.pc = opc
 
     @mutates
-    def handle_goto(self, args: List[str]) -> None:
+    def handle_goto(self, args: "List[str]") -> None:
         """
         goto <loc>
           Jump to the given location (either a line number or a label) without
@@ -496,7 +511,7 @@ class Shell:
 
         self.print_current_op()
 
-    def handle_help(self, args: List[str]) -> None:
+    def handle_help(self, args: "List[str]") -> None:
         """
         help
           Print a summary of all debugging commands.
@@ -522,7 +537,7 @@ class Shell:
                 if i != len(args) - 1:
                     print()
 
-    def handle_info(self, args: List[str]) -> None:
+    def handle_info(self, args: "List[str]") -> None:
         """
         info <arg>...
           Print information about the current state of the program. Valid arguments to
@@ -554,7 +569,7 @@ class Shell:
             if i != len(fullargs) - 1:
                 print()
 
-    def handle_list(self, args: List[str]) -> None:
+    def handle_list(self, args: "List[str]") -> None:
         """
         list
           Print the current line of source and the three previous and next lines.
@@ -578,7 +593,7 @@ class Shell:
         else:
             print("Program has finished executing.")
 
-    def handle_ll(self, args: List[str]) -> None:
+    def handle_ll(self, args: "List[str]") -> None:
         """
         ll
           Print every line of the current file's source code.
@@ -594,7 +609,7 @@ class Shell:
             print("Program has finished executing.")
 
     @mutates
-    def handle_next(self, args: List[str]) -> None:
+    def handle_next(self, args: "List[str]") -> None:
         """
         next
           Execute the current line. If the current line is a CALL instruction, the
@@ -625,7 +640,7 @@ class Shell:
         self.print_current_op()
 
     @mutates
-    def handle_off(self, args: List[str]) -> None:
+    def handle_off(self, args: "List[str]") -> None:
         """
         off <f1> <f2>...
           Turn off all the HERA machine flags listed. Flags may be given in long
@@ -646,7 +661,7 @@ class Shell:
             setattr(self.debugger.vm, flag, False)
 
     @mutates
-    def handle_on(self, args: List[str]) -> None:
+    def handle_on(self, args: "List[str]") -> None:
         """
         on <f1> <f2>...
           Turn on all the HERA machine flags listed. Flags may be given in long form
@@ -749,7 +764,7 @@ class Shell:
             print(self.format_int(value, spec))
 
     @mutates
-    def handle_restart(self, args: List[str]) -> None:
+    def handle_restart(self, args: "List[str]") -> None:
         """
         restart
           Restart execution of the program from the beginning. All registers and
@@ -763,7 +778,7 @@ class Shell:
         self.print_current_op()
 
     @mutates
-    def handle_step(self, args: List[str]) -> None:
+    def handle_step(self, args: "List[str]") -> None:
         """
         step
           Step into the execution of a function.  The step command is only valid when
@@ -780,7 +795,7 @@ class Shell:
         self.debugger.next(step=True)
         self.print_current_op()
 
-    def handle_undo(self, args: List[str]) -> None:
+    def handle_undo(self, args: "List[str]") -> None:
         """
         undo
           Undo the last operation that changed the state of the debugger.
@@ -905,7 +920,7 @@ class Shell:
         """
         vm = self.debugger.vm
         if isinstance(node, IntNode):
-            if node.value >= 2 ** 16 or node.value < -2 ** 15:
+            if node.value >= 2 ** 16 or node.value < -(2 ** 15):
                 raise HERAError("integer literal exceeds 16 bits")
             return node.value
         elif isinstance(node, RegisterNode):
@@ -966,7 +981,9 @@ class Shell:
         else:
             print("Program has finished executing.")
 
-    def print_range_of_ops(self, loc: Location, context: Optional[int] = None) -> None:
+    def print_range_of_ops(
+        self, loc: Location, context: "Optional[int]" = None
+    ) -> None:
         """
         Print the line indicated by `loc`, as well as `context` previous and following
         lines. If `context` is None, the whole file is printed.
@@ -1110,6 +1127,6 @@ Available commands:
 
     <x> = <y>       Alias for "assign <x> <y>".
 
-Command names can generally be abbreviated with a unique prefix, e.g. "n" for
-"next".
-"""
+Commands can be abbreviated with a unique prefix, e.g. "n" for "next".
+
+Enter "help <command>" for detailed help on a specific command."""
